"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.prependForwardSlash = exports.makeUrlPath = exports.isNotForwardSlash = exports.isEmpty = exports.getWithContentTypeCsv = exports.getFn = exports.getAsCSV = exports.formatUrlPath = exports.addSearchQuery = exports.acceptCsvFiletype = void 0;

var _axios = _interopRequireDefault(require("axios"));

var _jsFileDownload = _interopRequireDefault(require("js-file-download"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const isNotForwardSlash = v => v && v !== '/';

exports.isNotForwardSlash = isNotForwardSlash;

const prependForwardSlash = v => {
  let output = String(v);
  if (output.startsWith('/')) output = output.substr(1);
  if (output.charAt(output.length - 1) === '/') output = output.substring(0, output.length - 1);
  return `/${output}`;
};

exports.prependForwardSlash = prependForwardSlash;

const getFn = (obj, prop) => {
  if (!(prop in obj) || typeof obj[prop] !== 'function') throw new Error('Unknown action');
  return obj[prop]();
};

exports.getFn = getFn;

const isEmpty = obj => !obj || typeof obj === 'object' && !Object.keys(obj).length && !(obj instanceof Error);

exports.isEmpty = isEmpty;

const makeUrlPath = (a, b) => {
  let url = Array.isArray(a) ? a.filter(isNotForwardSlash).map(prependForwardSlash).join('') : a;
  if (Array.isArray(b)) url += `?ids[]=${b.join('&ids[]=')}`;
  return url;
};

exports.makeUrlPath = makeUrlPath;

const addSearchQuery = (v, term) => v.includes('?') ? `${v}&search=${term}` : `${v}?search=${term}`;

exports.addSearchQuery = addSearchQuery;

const acceptCsvFiletype = (params = {}) => (data, headers) => {
  Object.assign(headers, params, {
    'Accept': 'text/csv'
  });
  return data;
};

exports.acceptCsvFiletype = acceptCsvFiletype;

const getWithContentTypeCsv = (url, params) => (0, _axios.default)({
  url,
  method: 'get',
  transformRequest: [acceptCsvFiletype(params)]
});

exports.getWithContentTypeCsv = getWithContentTypeCsv;

const getAsCSV = (url, params = {}) => getWithContentTypeCsv(url, params).then(e => (0, _jsFileDownload.default)(e.data, 'file.csv')).catch(() => {// noop
});

exports.getAsCSV = getAsCSV;

const formatUrlPath = (url, query, select) => {
  let endpoint = url;

  const hasQuery = v => v.includes('?');

  const appendAmpersand = v => v.startsWith('&') ? v : `&${v}`;

  const hasLength = v => typeof v === 'string' && v.length;

  const addToEndpoint = v => {
    if (hasQuery(endpoint)) {
      endpoint += appendAmpersand(v.replace('?', '&'));
    } else if (v.startsWith('&')) {
      endpoint += endpoint.endsWith('&') ? v.substr(1) : v;
    } else {
      endpoint += !v.startsWith('?') ? `?${v}` : v;
    }
  };

  if (hasLength(query)) addToEndpoint(query);
  if (hasLength(select)) addToEndpoint(`fields=${select}`);
  return endpoint;
};

exports.formatUrlPath = formatUrlPath;