"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.mergeUniq = exports.enforceQueryString = exports.default = void 0;

var _react = _interopRequireDefault(require("react"));

var _lodash = require("lodash");

var _q3UiHelpers = require("q3-ui-helpers");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const mergeUniq = (a, b) => (0, _lodash.uniqBy)((0, _lodash.compact)([a, b].flat(2)), 'id');

exports.mergeUniq = mergeUniq;

const enforceQueryString = str => !String(str).startsWith('?') && (0, _lodash.size)(str) ? `?${str}` : str;

exports.enforceQueryString = enforceQueryString;

const useInfiniteScroll = ({
  data,
  hasNextPage = false,
  poll,
  location = {}
}) => {
  const ref = _react.default.useRef(); // assumes useRest already ran for 0


  const initialPageValue = 1;

  const [page, setPage] = _react.default.useState(initialPageValue);

  const [cache, setCache] = _react.default.useState([]);

  const [stop, setStop] = _react.default.useState(false);

  const q = enforceQueryString(location === null || location === void 0 ? void 0 : location.search);

  _react.default.useLayoutEffect(() => {
    setCache(mergeUniq(data, cache));
  }, [data]);

  _react.default.useLayoutEffect(() => {
    setCache([]); // different data set altogether

    setPage(initialPageValue);
    setStop(false);
  }, [q]);

  _react.default.useEffect(() => {
    let observer;
    let timer;

    const runObserver = method => {
      if (ref.current) (0, _lodash.invoke)(observer, method, ref.current);
    };

    const disconnect = () => {
      try {
        runObserver('unobserve');
        clearTimeout(timer);
      } catch (e) {// noop
      }
    };

    observer = new IntersectionObserver(entries => {
      var _first;

      if (((_first = (0, _lodash.first)(entries)) === null || _first === void 0 ? void 0 : _first.isIntersecting) === true) {
        disconnect();

        _q3UiHelpers.object.noop(poll((0, _lodash.compact)([q, `page=${page}`]).join('&')).then(r => {
          if ((r === null || r === void 0 ? void 0 : r.hasNextPage) === false) {
            setStop(true);
          }

          setPage(page + 1);
        }));
      }
    }, {
      threshold: [0]
    });
    timer = setTimeout(() => {
      runObserver('observe');
    }, 2500);
    return disconnect;
  }, [page, q]);

  return {
    data: cache,
    scrollWatchRef: ref,
    showScrollWatch: !stop && hasNextPage
  };
};

var _default = useInfiniteScroll;
exports.default = _default;