"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.convertUtcDateStringsToLocalTime = convertUtcDateStringsToLocalTime;
exports.serializeDateFromUtcToLocalTime = exports.default = void 0;

var _axios = _interopRequireDefault(require("axios"));

var _qs = _interopRequireDefault(require("qs"));

var _flat = _interopRequireDefault(require("flat"));

var _q3UiLocale = require("q3-ui-locale");

var _react = _interopRequireDefault(require("react"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const serializeDateFromUtcToLocalTime = (config = {}) => {
  const newConfig = { ...config
  };

  newConfig.paramsSerializer = params => _qs.default.stringify(params, {
    serializeDate: _q3UiLocale.timezone.toUtc
  });

  return newConfig;
};

exports.serializeDateFromUtcToLocalTime = serializeDateFromUtcToLocalTime;

function convertUtcDateStringsToLocalTime(config = {}) {
  try {
    const flattened = Object.entries((0, _flat.default)(config.data)).reduce((acc, [key, value]) => {
      acc[key] = _q3UiLocale.timezone.isUtc(value) ? _q3UiLocale.timezone.toLocal(value) : value;
      return acc;
    }, {}); // eslint-disable-next-line

    config.data = _flat.default.unflatten(flattened);
    return config;
  } catch (e) {
    return config;
  }
}

const useUTCToLocalInterceptors = preferredTimezone => {
  _q3UiLocale.timezone.setTimezone(preferredTimezone);

  _react.default.useLayoutEffect(() => {
    const req = _axios.default.interceptors.request.use(serializeDateFromUtcToLocalTime);

    const res = _axios.default.interceptors.response.use(convertUtcDateStringsToLocalTime);

    return () => {
      _axios.default.interceptors.request.eject(req);

      _axios.default.interceptors.response.eject(res);
    };
  }, []);
};

var _default = useUTCToLocalInterceptors;
exports.default = _default;